//=============================================================================
// error processing
//=============================================================================

#include "errors.h"

#include "lists.h"
#include "streams.h"
#include "assemble.h"


//-----------------------------------------------------------------------------
//---= error-tabel =-----------------------------------------------------------
//-----------------------------------------------------------------------------

char *err_tbl[_RETC_LAST] =
{
	/*RETC_UNKNOWN*/		"(unknown)",

	/*RETC_DIV_BY_ZERO*/	"divide by zero",
	/*RETC_TOO_MANY_CALLS*/	"too many calls without termination",

	/*RETC_UNKWN_CMDPARAM*/	"error in command line (use option -? to get info)\n",

	/*RETC_TOO_MANY_ERR*/	"internal : too much errors occured",
	/*RETC_TOO_MANY_FILES*/	"internal : too many files open",
	/*RETC_OUTOFMEM*/		"internal : out of memory",
	/*RETC_LISTALLOC*/		"internal : could not allocate memory for list",
	/*RETC_LISTREQUEST*/	"internal : request for list failed",

	/*RETC_FOPEN*/			"internal : could not open file",
	/*RETC_FCREATE*/		"internal : could not create file for binary output",
	/*RETC_FREAD*/			"internal : could not read from file",
	/*RETC_FWRITE*/			"internal : could not write to output file",
	/*RETC_FACCESS*/		"internal : error accessing file",
	/*RETC_LINE2LONG*/		"internal : too much characters in line",

	/*RETC_INCLUDE_ERR*/	"error in include file",
	/*RETC_INCLUDE_ABORT*/	"could not include file",
	/*RETC_MACRO_ERR*/		"error in macro",
	/*RETC_MACRO_ABORT*/	"could not execute macro",
	/*RETC_IFELSE_ERR*/		"error in if / else directive",
	/*RETC_IFELSE_ABORT*/	"could not execute if / else",

	/*RETC_PASS_DEPENDENT*/	/*code is pass dependent*/ "compatibility pass expected",

	/*RETC_NC_REGION*/		"jump not in current region",
	/*RETC_TARG_MISMATCH*/	"instruction not defined in selected target",
	/*RETC_UNKWN_INSTR*/	"unknown instruction",
	/*RETC_UNKWN_LBL_SYNTAX*/ "syntax error in label",
	/*RETC_KEY_OVERWRITE*/	"overwrites keyword",
	/*RETC_LBL_OVERWRITE*/	"overwrites label",
	/*RETC_MACRO_OVERWRITE*/	"overwrites macro",
	/*RETC_MARK_MISSING*/	"mark is missing",
	/*RETC_PARAM_EXPECTED*/	"parameter expected",
	/*RETC_UNKWN_INSTR_SYNTAX*/	"unknown instruction syntax",
	/*RETC_DELIMITER_EXPECTED*/ "no COMMENT-end delimiter in this file",

	/*RETC_BRACKET_OPEN*/	"bracket not closed",
	/*RETC_BRACKET_CLOSE*/	"bracket closed, but not opened",
	/*RETC_OVERFLOW*/		"value exceeded 64bit signed",
	/*RETC_SYNTAX_ERR*/		"syntax error",
	/*RETC_TOO_MUCH_PARAMS*/ "too much parameters",
	/*RETC_TARG_ALRD_SEL*/	"target already selected",

	/*RETC_NO_LBLS_ALLOWED*/ "labels not allowed as parameter",

	/*RETC_REG_OVRFLW*/		"register overflow",
							"register overflow in param 1","register overflow in param 2","register overflow in param 3",
	/*RETC_IMM_OVRFLW*/		"immediate overflow",
							"immediate overflow in param 1","immediate overflow in param 2","immediate overflow in param 3",
	/*RETC_OFF_OVRFLW*/		"offset overflow",
							"offset overflow in param 1","offset overflow in param 2","offset overflow in param 3",
	/*RETC_OFF_INVALID*/	"no valid offset",
							"no valid offset in param 1","no valid offset in param 2","no valid offset in param 3",
	/*RETC_TARG_OVRFLW*/	"target overflow",
							"target overflow in param 1","target overflow in param 2","target overflow in param 3",
	/*RETC_TARG_INVALID*/	"no accessable target",
							"no accessable target in param 1","no accessable target in param 2","no accessable target in param 3",
	/*RETC_CODE_OVRFLW*/	"value overflow",
							"value overflow in param 1","value overflow in param 2","value overflow in param 3",
	/*RETC_WRONG_PTYPE*/	"wrong parameter type",
							"wrong parameter type in param 1","wrong parameter type in param 2","wrong parameter type in param 3",

	/*RETC_ELSE_WO_IF*/		"ELSE without IF directive",
	/*RETC_ENDIF_WO_IF*/	"ENDIF without IF directive",
	/*RETC_ENDIF_EXPECTED*/	"ENDIF statement missing",
	/*RETC_MACRO_EXPECTED*/	"ENDM without MACRO directive",
	/*RETC_ENDM_EXPECTED*/	"MACRO without ENDM directive",
	/*RETC_TABLE_EXPECTED*/	"ENDT without TABLE directive",
	/*RETC_ENDT_EXPECTED*/	"TABLE without ENDT directive",
	/*RETC_PROC_EXPECTED*/	"ENDP without PROC directive",
	/*RETC_ENDP_EXPECTED*/	"ENDP expected",
	/*RETC_SEG_EXPECTED*/	"ENDS without SEGMENT directive",
	/*RETC_ENDS_EXPECTED*/	"ENDS expected",

	/*RETC_ERR_IN_MSG*/		"one or more params of message invalid",

    /*RETC_LABEL_UNDEF*/	"label not defined",
	/*RETC_EXTRA_CHAR*/		"extra characters on line",
	/*RETC_OVRWR_ASSIGN*/	"overwrites assignment",
	/*RETC_OVRWR_EOF*/		"overwrites end of code statement"
};


//-----------------------------------------------------------------------------
//---= data =------------------------------------------------------------------
//-----------------------------------------------------------------------------
DWORD error_cntr=0;		//holds numbers of occured errors
DWORD warning_cntr=0;	//holds numbers of occured warnings

FILE *ferr = NULL;		//handle 4 seccond error output


//-----------------------------------------------------------------------------
// do_umsg: does output msg to screen and if available to file
//-----------------------------------------------------------------------------
void do_umsg(char *msg)
{
	//if(err_hide) return;
	printf("%s", msg);
	if(ferr) fprintf(ferr, "%s", msg);
}


//-----------------------------------------------------------------------------
// do_error: do error-output and return (WARNING/ERROR) or terminate (FERR)
//-----------------------------------------------------------------------------
void do_error(DWORD code, BYTE flags)
{
	dbgout("\ndo_error:\n")
	FILELIST *fl = NULL;

	if( (!assemble.preprocess) &&
	((error_cntr+warning_cntr) >= MAX_ERRORS)&&(code != (ERR_FATAL|RETC_TOO_MANY_ERR)) )
	{
		code = ERR_FATAL|RETC_TOO_MANY_ERR;
		flags = ERR_SHOW_NOTHING;
	}

	//process errors only in the last step (except FATAL)
	if((stream.file_no != -1)&&(assemble.preprocess))
	switch(code & ERR_TYPE_MASK)
	{
	case ERR_FATAL:
		break;

	//case ERR_WARNING:
	//	warning_cntr++;
	//	return;

	//case ERR_NORMAL:
	default:
	//	error_cntr++;
		return;
	}
		
	//first test flags for show-cmds
	if(flags!=ERR_SHOW_NOTHING)
	{
		if(stream.file_no == -1)
		{
			dbgout("[internal] : ")
			if(ferr) fprintf(ferr, "[internal] : ");
			else printf("[internal] : ");
		}
		else
		{
			if(!(fl = stream.get())) do_error(ERR_FATAL|RETC_LISTREQUEST, ERR_SHOW_NOTHING);

			switch(flags)
			{
			case ERR_SHOW_FILE:
				dbgout(fl->name << " : ")
				if(ferr) fprintf(ferr, "%s : ", fl->name);
				else printf("%s : ", fl->name);
				break;

			case ERR_SHOW_LINE:
				dbgout("line(" << fl->line_cntr << ") : ")
				if(ferr) fprintf(ferr, "line(%i) : ", fl->line_cntr);
				else printf("line(%i) : ", fl->line_cntr);
				break;

			default: //ERR_SHOW_POS
				dbgout(fl->name << "(" << fl->line_cntr << ") : ")
				if(ferr) fprintf(ferr, "%s(%i) : ", fl->name, fl->line_cntr);
				else printf("%s(%i) : ", fl->name, fl->line_cntr);
			}
		}
	}

	//decide which error type
	switch(code & ERR_TYPE_MASK)
	{
	case ERR_FATAL:
		error_cntr++;
		if(fl) fl->error_cntr++;

		dbgout("fatal error " << code << ": ")
		printf("fatal error %05X: ", code);
		if(ferr) fprintf(ferr, "fatal error %05X: ", code);

		code=code & ERR_CODE_MASK;
		if(code>=_RETC_LAST) code=RETC_UNKNOWN;
		
		dbgout(err_tbl[code] << "\n\n")
		printf("%s\n", err_tbl[code]);
		if(ferr) fprintf(ferr, "%s\n", err_tbl[code]);
		
		if(ferr) fclose(ferr);
		
		//show info
		if(assemble.preprocess)
		{
			printf("fatal error occured, preprocessing aborted\n");
			if(ferr) fprintf(ferr, "fatal error occured, preprocessing aborted\n");
		}
		else
		{
			fl = stream.get(0);
			if((fl)&&(fl->name)&&(*fl->name))
			{
				printf("%s - ", fl->name);
				if(ferr) fprintf(ferr, "%s - ", fl->name);
			}
			//else
			//{
			//	printf("\n");
			//	if(ferr) fprintf(ferr, "\n");
			//}
			
			printf("fatal error occured, assembling aborted\n");
			if(ferr) fprintf(ferr, "fatal error occured, assembling aborted\n");
		}

		_fcloseall();
		_exit(-1);

	case ERR_WARNING:
		warning_cntr++;
		if(fl) fl->warning_cntr++;

		dbgout("warning " << code << ": ")
		if(ferr) fprintf(ferr, "warning %05X: ", code);
		else printf("warning %05X: ", code);

		code=code & ERR_CODE_MASK;
		if(code>=_RETC_LAST) code=RETC_UNKNOWN;
		
		dbgout(err_tbl[code] << "\n\n")
		if(ferr) fprintf(ferr, "%s\n", err_tbl[code]);
		else printf("%s\n", err_tbl[code]);
		return;

	//case ERR_NORMAL:
	default:
		error_cntr++;
		if(fl) fl->error_cntr++;

		code=code & ERR_CODE_MASK;
		dbgout("error " << (code|ERR_NORMAL) << ": ")
		if(ferr) fprintf(ferr, "error %05X: ", code|ERR_NORMAL);
		else printf("error %05X: ", code|ERR_NORMAL);

		if(code>=_RETC_LAST) code=RETC_UNKNOWN;
		
		dbgout(err_tbl[code] << "\n\n")
		if(ferr) fprintf(ferr, "%s\n", err_tbl[code]);
		else printf("%s\n", err_tbl[code]);
	}
}