//=============================================================================
// 32=bit assembler for ps2 risc5900mips by wiRe (01/21/2001)
//=============================================================================

#include "main.h"


//-----------------------------------------------------------------------------
// classes
//-----------------------------------------------------------------------------
#ifdef _DEBUG
	ofstream ofdbg;
#endif


ASSEMBLE assemble;
INSTRUCTION instruction;
DIRECTIVE directive;
INTMATH intmath;
MACRO macro;
STREAM stream;
ASMCLASS asmclass;
LABEL label;


const char _DEFAULT_ERRLOG[] = "errors.log";
const char _PSX_[] = "psx";
const char _SCE_[] = "sce";
const char _ECO_[] = "eco";
const char _CPE_[] = "cpe";
const char _BIN_[] = "bin";
const char _ELF_[] = "elf";


//-----------------------------------------------------------------------------
// main
//-----------------------------------------------------------------------------
int main(int argc, char *argv[], char *envp[])
{
	printf("asm5900 v1.8b: ps2 risc5900 mips assembler by wiRe (20/01/2001)\n");
	printf("http://napalm.intelinet.com/~wiRe/?projects.asm5900\n");

	#ifdef _DEBUG
		printf("DEBUG-VERSION (output goes 2 DBGLOG.TXT)!\n");
		ofdbg.setbuf(NULL, 0);
		ofdbg.open( "DBGLOG.TXT", ios::trunc );
	#endif

	//scan parameters
	char *fin=NULL;
	char *fout=NULL;
	char *flog=NULL;
	BOOL targ=false;
	IM_RET imr;


	if(argc<=1) do_error(ERR_FATAL|RETC_UNKWN_CMDPARAM, ERR_SHOW_NOTHING);
	for(int cntr=1; cntr<argc; cntr++)
	{
		_strlwr(argv[cntr]);
		if((*argv[cntr]=='/')||(*argv[cntr]=='-'))	//switch??
		{
			switch(*(++argv[cntr]))
			{
			case '?':
			case 'h':
				printf( "syntax: asm5900 source [/o{:<file>}] [/e{:<file>}] [options]\n" \
						"\n" \
						"/d{:<file>}\t\twrite debug info to file (def: *.dbg)\n" \
						"/e{:<file>}\t\tlog screen output to file (def: errors.log)\n" \
						"/l{:<file>}\t\twrite listing to file (def: *.lst)\n" \
						"/o{:<file>}\t\twrite assembled code to file (def: *.<target>)\n" \
						"/t:<target>\t\tselect target (PSX,SCE,ECO,CPE,BIN,def: BIN)\n" \
					    "\n" \
						"/$<-|+>\t\t\tuse $ to define hex numbers (def: $-)\n" \
						"/c<+|->\t\t\ttoggle case sensitive mode (def: c+)\n" \
						"/g<-|+>\t\t\ttoggle global label adressing mode (def: g-)\n" \
						"/r<+|->\t\t\tload default register values (def: r+)\n" \
						"\n" \
						"/i:<path>\t\tsets search path for include files\n" \
						"/m:<value>\t\tsets maximum number of passes (2..99, def: 4)\n" \
						"/s:<symbol>{=<value>}\tdefines a symbol (true on defaul)\n" \
						"/w<-|+>{<warnclass>}\tcontrols the generation of warnings\n" \
						"\n" \
						"/h, /?\t\t\tdisplay this help screen\n" \
						""
				);
				return(0);

			//-----------------------------------------------------------------------------------------

			case 'd':
				if(!assemble.config.dbg_file)
				{
					if(*(++argv[cntr])==':')	//get filename
					{
						assemble.config.dbg_file = ++argv[cntr];
						break;
					}
					else if(!*argv[cntr]) //otherwise use default
					{
						assemble.config.dbg_file = (char *)-1;
						break;
					}
				}
				do_error(ERR_FATAL|RETC_UNKWN_CMDPARAM, ERR_SHOW_NOTHING);
			
			case 'e':
				if(!flog)
				{
					if(*(++argv[cntr])==':')	//get filename
					{
						flog = ++argv[cntr];
						break;
					}
					else if(!*argv[cntr]) //otherwise use default
					{
						flog=(char *)malloc(sizeof(_DEFAULT_ERRLOG)+1);
						if(!flog) do_error(ERR_FATAL|RETC_OUTOFMEM, 0);
						strcpy(flog, _DEFAULT_ERRLOG);
						break;
					}
				}
				do_error(ERR_FATAL|RETC_UNKWN_CMDPARAM, ERR_SHOW_NOTHING);
			
			case 'l':
				if(!assemble.config.lst_file)
				{
					if(*(++argv[cntr])==':')	//get filename
					{
						assemble.config.lst_file = ++argv[cntr];
						break;
					}
					else if(!*argv[cntr]) //otherwise use default
					{
						assemble.config.lst_file = (char *)-1;
						break;
					}
				}
				do_error(ERR_FATAL|RETC_UNKWN_CMDPARAM, ERR_SHOW_NOTHING);
			
			case 'o':
				if(!fout)
				{
					if(*(++argv[cntr])==':')	//get filename
					{
						fout = ++argv[cntr];
						break;
					}
					else if(!*argv[cntr]) //otherwise use default
					{
						fout = (char *)-1;
						break;
					}
				}
				do_error(ERR_FATAL|RETC_UNKWN_CMDPARAM, ERR_SHOW_NOTHING);

			case 't':
				if((*(++argv[cntr]) == ':') && *(++argv[cntr]))
				{
					if(assemble.config.target != SEL_TARG_DEFAULT)
						do_error(ERR_FATAL|RETC_TARG_ALRD_SEL, ERR_SHOW_NOTHING);

					if(strcmp(argv[cntr], _PSX_) == 0) {assemble.config.target = SEL_TARG_PSX; assemble.binary = false; break;}
					if(strcmp(argv[cntr], _SCE_) == 0) {assemble.config.target = SEL_TARG_SCE; assemble.binary = false; break;}
					if(strcmp(argv[cntr], _ECO_) == 0) {assemble.config.target = SEL_TARG_ECO; assemble.binary = false; break;}
					if(strcmp(argv[cntr], _CPE_) == 0) {assemble.config.target = SEL_TARG_CPE; assemble.binary = false; break;}
					if(strcmp(argv[cntr], _BIN_) == 0) {assemble.config.target = SEL_TARG_BIN; assemble.binary = true; break;}
					if(strcmp(argv[cntr], _ELF_) == 0) {assemble.config.target = SEL_TARG_ELF; assemble.binary = false; break;}
				}
				do_error(ERR_FATAL|RETC_UNKWN_CMDPARAM, ERR_SHOW_NOTHING);

			//-----------------------------------------------------------------------------------------

			case '$':
				argv[cntr]++;
				switch(*(WORD*)argv[cntr])
				{
				case '+': intmath.enable_$ = true; break;
				case '-': intmath.enable_$ = false; break;
				default: do_error(ERR_FATAL|RETC_UNKWN_CMDPARAM, ERR_SHOW_NOTHING);
				}
				break;
			
			case 'c':
				argv[cntr]++;
				switch(*(WORD*)argv[cntr])
				{
				case '+': assemble.config.case_sensitive = true; break;
				case '-': assemble.config.case_sensitive = false; break;				
				default: do_error(ERR_FATAL|RETC_UNKWN_CMDPARAM, ERR_SHOW_NOTHING);
				}
				break;

			case 'g':
				argv[cntr]++;
				switch(*(WORD*)argv[cntr])
				{
				case '+': assemble.config.default_local = false; break;
				case '-': assemble.config.default_local = true; break;
				default: do_error(ERR_FATAL|RETC_UNKWN_CMDPARAM, ERR_SHOW_NOTHING);
				}
				break;

			case 'r':
				argv[cntr]++;
				switch(*(WORD*)argv[cntr])
				{
				case '+': assemble.config.default_define = true; break;
				case '-': assemble.config.default_define = false; break;
				default: do_error(ERR_FATAL|RETC_UNKWN_CMDPARAM, ERR_SHOW_NOTHING);
				}
				break;

			//-----------------------------------------------------------------------------------------

			case 'm':
				imr.str = argv[cntr]+1;
				if((intmath.calc(&imr))&&(imr.value>=2)&&(imr.value<=99))
				{
				  assemble.config.max_pass = (int)imr.value;
				  break;
				}
			    do_error(ERR_FATAL|RETC_UNKWN_CMDPARAM, ERR_SHOW_NOTHING);

			default:
				do_error(ERR_FATAL|RETC_UNKWN_CMDPARAM, ERR_SHOW_NOTHING);

			}
		}
		else if(*argv[cntr])	//infile???
		{
			if(fin) do_error(ERR_FATAL|RETC_TOO_MUCH_PARAMS, ERR_SHOW_NOTHING);
			fin = argv[cntr];
		}
	}


	char *dot;

	if(!fin) do_error(ERR_FATAL|RETC_UNKWN_CMDPARAM, ERR_SHOW_NOTHING);

	if(fout == (char *)-1)	//define default outputfile
	{
		fout=(char *)malloc(strlen(fin)+5);
		if(!fout) do_error(ERR_FATAL|RETC_OUTOFMEM, 0);
		strcpy(fout, fin);
		dot = strrchr(fout, '.');
		if(dot) *dot = NULL;
	}

	if(assemble.config.dbg_file == (char *)-1)	//then calc default name
	{
		assemble.config.dbg_file = (char *)malloc(strlen(fin)+5);
		if(!assemble.config.dbg_file) do_error(ERR_FATAL|RETC_OUTOFMEM, 0);
		strcpy(assemble.config.dbg_file, fin);
		dot = strrchr(assemble.config.dbg_file, '.');
		if(!dot) strcat(assemble.config.dbg_file, ".dbg");
		else strcpy(dot, ".dbg");
	}
	if(assemble.config.dbg_file)
		printf("write debug info to file: %s\n", assemble.config.dbg_file);

	if(assemble.config.lst_file == (char *)-1)	//then calc default name
	{
		assemble.config.lst_file = (char *)malloc(strlen(fin)+5);
		if(!assemble.config.lst_file) do_error(ERR_FATAL|RETC_OUTOFMEM, 0);
		strcpy(assemble.config.lst_file, fin);
		dot = strrchr(assemble.config.lst_file, '.');
		if(!dot) strcat(assemble.config.lst_file, ".lst");
		else strcpy(dot, ".lst");
	}
	if(assemble.config.lst_file)
		printf("write listing to file: %s\n", assemble.config.lst_file);

	if(flog)
	{
		printf("error output goes to file: %s\n", flog);
		ferr = fopen(flog, "w");
		if(!ferr) do_error(ERR_FATAL|RETC_FCREATE, 0);
	}

	if(!fout)	//warn user
	{
		printf("no output file selected: virtual mode\n");
		if(ferr) fprintf(ferr, "no output file selected: virtual mode\n\n");
	}

	if(!ferr) printf("\n");

	//start assembling NOW!
	assemble.now(fin, fout);

	return(0);
}
