//opcodes of the r5900 mips
#ifndef _OPCODES_
#define _OPCODES_


#define NUM_OPCODES 468


//used params
#define PARAM_RD_RS_RT 1
#define PARAM_RT_RS_IMM 2
#define PARAM_RS_RT_OFF 3
#define PARAM_RS_OFF 4
#define PARAM_NOTHING 5
#define PARAM_CODE 6
#define PARAM_RS_RT 7
#define PARAM_RD_RT_SA 8
#define PARAM_RD_RT_RS 9
#define PARAM_TARG 10
#define PARAM_RS 11
#define PARAM_RD_RS 12
#define PARAM_RT_OFF_BASE_ 13
#define PARAM_RT_NOBASE 14
#define PARAM_RT_IMM 15
#define PARAM_RD 16
#define PARAM_HINT_OFF_BASE_ 17
#define PARAM_HINT_NOBASE 18
#define PARAM_RS_IMM 19
#define PARAM_RT_RS_INV 20
#define PARAM_RD_RT 21
#define PARAM_OFF 22
#define PARAM_OP_OFF_BASE_ 23
#define PARAM_OP_NOBASE 24
#define PARAM_RT 25
#define PARAM_RT_RD 26
#define PARAM_RT_REG 27
#define PARAM_FD_FS 28
#define PARAM_FD_FS_FT 29
#define PARAM_FS_FT 30
#define PARAM_RT_FS 31
#define PARAM_FT_OFF_BASE_ 32
#define PARAM_FT_NOBASE 33
#define PARAM_FD_FT 34
#define PARAM_RT_ID 35
#define PARAM_RT_FD 36
#define PARAM_COFUN 37

//used typesets
#define TYPESET_RTYPE 1
#define TYPESET_ITYPE 2
#define TYPESET_CTYPE 3
#define TYPESET_JTYPE 4
#define TYPESET_BTYPE 5
#define TYPESET_HTYPE 6
#define TYPESET_ETYPE 7

//used targets
#define TARGET_R3000_R5900 1
#define TARGET_R5900 2
#define TARGET_R3000 3

//used param shift-counter
#define _PARAM(n,t) ((DWORD)(pl[n])->val<<t)
#define _NOBASE 0
#define _COFUN 0
#define _TARG 0
#define _CODE 6
#define _HINT 16
#define _BASE 21
#define _IMM 0
#define _INV 0
#define _OFF 0
#define _REG 1
#define _SA 6
#define _FD 6
#define _RD 11
#define _FS 11
#define _ID 11
#define _RT 16
#define _OP 16
#define _FT 16
#define _RS 21


typedef struct
{
	char	name[10];
	__int8	pnum;
	__int8	param;
	__int8	target;
	//__int8	typeset;
	DWORD	opcode;
} OPCODE;


OPCODE opcodes[NUM_OPCODES] =
{
//-----------------------------------------------------------
//   rs	5bit	register (source)
//   rd	5bit	register (destination)
//   rt	5bit	register (target=src v dest)
//   sa	5bit	shift amount
// base	5bit	register (with baseaddr)
//				gp_reg = 11100 is default base
// hint	5bit	any value (default is 0)
//  imm	16bit	any value (immediate)
//  inv  16bit	any value to negate (negative immediate)
//  off	16bit	any addr-value
// code	20bit	any value passes to system
// targ	26bit	any code-addr-value (is shifted 2 bits)
//cofun	25bit	coprocessor operation (r3000)
//
//	op	5bit	COP0 opcode register
//	fs	5bit	FPU fixed point register (source)
//	fd	5bit	FPU fixed point register (destination)
//	ft	5bit	FPU fixed point register (target=src v dest)
//	id	5bit	COP2 control register
//
//
//   -	no params
//-----------------------------------------------------------
//[MIPS]
	{"add", 3, PARAM_RD_RS_RT, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000020},
	{"addi", 3, PARAM_RT_RS_IMM, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x20000000},
	{"addiu", 3, PARAM_RT_RS_IMM, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x24000000},
	{"addu", 3, PARAM_RD_RS_RT, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000021},
	{"and", 3, PARAM_RD_RS_RT, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000024},
	{"andi", 3, PARAM_RT_RS_IMM, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x30000000},
	{"beq", 3, PARAM_RS_RT_OFF, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x10000000},
	{"beql", 3, PARAM_RS_RT_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x50000000},
	{"bgez", 2, PARAM_RS_OFF, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x04010000},
	{"bgezal", 2, PARAM_RS_OFF, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x04110000},
	{"bgezall", 2, PARAM_RS_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x04130000},
	{"bgezl", 2, PARAM_RS_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x04030000},
	{"bgtz", 2, PARAM_RS_OFF, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x1C000000},
	{"bgtzl", 2, PARAM_RS_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x5C000000},
	{"blez", 2, PARAM_RS_OFF, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x18000000},
	{"blezl", 2, PARAM_RS_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x58000000},
	{"bltz", 2, PARAM_RS_OFF, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x04000000},
	{"bltzal", 2, PARAM_RS_OFF, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x04100000},
	{"bltzall", 2, PARAM_RS_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x04120000},
	{"bltzl", 2, PARAM_RS_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x04020000},
	{"bne", 3, PARAM_RS_RT_OFF, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x14000000},
	{"bnel", 3, PARAM_RS_RT_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x54000000},
	{"break", 0, PARAM_NOTHING, TARGET_R3000_R5900, /*TYPESET_CTYPE,*/ 0x0000000D},
	{"break", 1, PARAM_CODE, TARGET_R3000_R5900, /*TYPESET_CTYPE,*/ 0x0000000D},
	{"dadd", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x0000002C},
	{"daddi", 3, PARAM_RT_RS_IMM, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x60000000},
	{"daddiu", 3, PARAM_RT_RS_IMM, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x64000000},
	{"daddu", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x0000002D},
	{"div", 2, PARAM_RS_RT, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x0000001A},
	{"divu", 2, PARAM_RS_RT, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x0000001B},
	{"dsll", 3, PARAM_RD_RT_SA, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x00000038},
	{"dsll32", 3, PARAM_RD_RT_SA, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x0000003C},
	{"dsllv", 3, PARAM_RD_RT_RS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x00000014},
	{"dsra", 3, PARAM_RD_RT_SA, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x0000003B},
	{"dsra32", 3, PARAM_RD_RT_SA, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x0000003F},
	{"dsrav", 3, PARAM_RD_RT_RS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x00000017},
	{"dsrl", 3, PARAM_RD_RT_SA, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x0000003A},
	{"dsrl32", 3, PARAM_RD_RT_SA, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x0000003E},
	{"dsrlv", 3, PARAM_RD_RT_RS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x00000016},
	{"dsub", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x0000002E},
	{"dsubu", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x0000002F},
	{"j", 1, PARAM_TARG, TARGET_R3000_R5900, /*TYPESET_JTYPE,*/ 0x08000000},
	{"jal", 1, PARAM_TARG, TARGET_R3000_R5900, /*TYPESET_JTYPE,*/ 0x0C000000},
	{"jalr", 1, PARAM_RS, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x0000F809},
	{"jalr", 2, PARAM_RD_RS, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000009},
	{"jr", 1, PARAM_RS, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000008},
	{"lb", 3, PARAM_RT_OFF_BASE_, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0x80000000},
	{"lb", 2, PARAM_RT_NOBASE, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0x83800000},
	{"lbu", 3, PARAM_RT_OFF_BASE_, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0x90000000},
	{"lbu", 2, PARAM_RT_NOBASE, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0x93800000},
	{"ld", 3, PARAM_RT_OFF_BASE_, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xDC000000},
	{"ld", 2, PARAM_RT_NOBASE, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xDF800000},
	{"ldl", 3, PARAM_RT_OFF_BASE_, TARGET_R5900, /*TYPESET_BTYPE,*/ 0x68000000},
	{"ldl", 2, PARAM_RT_NOBASE, TARGET_R5900, /*TYPESET_BTYPE,*/ 0x6B800000},
	{"ldr", 3, PARAM_RT_OFF_BASE_, TARGET_R5900, /*TYPESET_BTYPE,*/ 0x6C000000},
	{"ldr", 2, PARAM_RT_NOBASE, TARGET_R5900, /*TYPESET_BTYPE,*/ 0x6F800000},
	{"lh", 3, PARAM_RT_OFF_BASE_, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0x84000000},
	{"lh", 2, PARAM_RT_NOBASE, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0x87800000},
	{"lhu", 3, PARAM_RT_OFF_BASE_, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0x94000000},
	{"lhu", 2, PARAM_RT_NOBASE, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0x97800000},
	{"lui", 2, PARAM_RT_IMM, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x3C000000},
	{"lw", 3, PARAM_RT_OFF_BASE_, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0x8C000000},
	{"lw", 2, PARAM_RT_NOBASE, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0x8F800000},
	{"lwl", 3, PARAM_RT_OFF_BASE_, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0x88000000},
	{"lwl", 2, PARAM_RT_NOBASE, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0x8B800000},
	{"lwr", 3, PARAM_RT_OFF_BASE_, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0x98000000},
	{"lwr", 2, PARAM_RT_NOBASE, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0x9B800000},
	{"lwu", 3, PARAM_RT_OFF_BASE_, TARGET_R5900, /*TYPESET_BTYPE,*/ 0x9C000000},
	{"lwu", 2, PARAM_RT_NOBASE, TARGET_R5900, /*TYPESET_BTYPE,*/ 0x9F800000},
	{"mfhi", 1, PARAM_RD, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000010},
	{"mflo", 1, PARAM_RD, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000012},
	{"movn", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x0000000B},
	{"movz", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x0000000A},
	{"mthi", 1, PARAM_RS, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000011},
	{"mtlo", 1, PARAM_RS, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000013},
	{"mult", 2, PARAM_RS_RT, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000018},
	{"multu", 2, PARAM_RS_RT, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000019},
	{"nop", 0, PARAM_NOTHING, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000000},
	{"nor", 3, PARAM_RD_RS_RT, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000027},
	{"or", 3, PARAM_RD_RS_RT, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000025},
	{"ori", 3, PARAM_RT_RS_IMM, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x34000000},
	{"pref", 3, PARAM_HINT_OFF_BASE_, TARGET_R5900, /*TYPESET_HTYPE,*/ 0xCC000000},
	{"pref", 2, PARAM_HINT_NOBASE, TARGET_R5900, /*TYPESET_HTYPE,*/ 0xCF800000},
	{"sb", 3, PARAM_RT_OFF_BASE_, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0xA0000000},
	{"sb", 2, PARAM_RT_NOBASE, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0xA3800000},
	{"sd", 3, PARAM_RT_OFF_BASE_, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xFC000000},
	{"sd", 2, PARAM_RT_NOBASE, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xFF800000},
	{"sdl", 3, PARAM_RT_OFF_BASE_, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xB0000000},
	{"sdl", 2, PARAM_RT_NOBASE, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xB3800000},
	{"sdr", 3, PARAM_RT_OFF_BASE_, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xB4000000},
	{"sdr", 2, PARAM_RT_NOBASE, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xB7800000},
	{"sh", 3, PARAM_RT_OFF_BASE_, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0xA4000000},
	{"sh", 2, PARAM_RT_NOBASE, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0xA7800000},
	{"sll", 3, PARAM_RD_RT_SA, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000000},
	{"sllv", 3, PARAM_RD_RT_RS, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000004},
	{"slt", 3, PARAM_RD_RS_RT, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x0000002A},
	{"slti", 3, PARAM_RT_RS_IMM, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x28000000},
	{"sltiu", 3, PARAM_RT_RS_IMM, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x2C000000},
	{"sltu", 3, PARAM_RD_RS_RT, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x0000002B},
	{"sra", 3, PARAM_RD_RT_SA, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000003},
	{"srav", 3, PARAM_RD_RT_RS, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000007},
	{"srl", 3, PARAM_RD_RT_SA, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000002},
	{"srlv", 3, PARAM_RD_RT_RS, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000006},
	{"sub", 3, PARAM_RD_RS_RT, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000022},
	{"subu", 3, PARAM_RD_RS_RT, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000023},
	{"sw", 3, PARAM_RT_OFF_BASE_, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0xAC000000},
	{"sw", 2, PARAM_RT_NOBASE, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0xAF800000},
	{"swl", 3, PARAM_RT_OFF_BASE_, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0xA8000000},
	{"swl", 2, PARAM_RT_NOBASE, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0xAB800000},
	{"swr", 3, PARAM_RT_OFF_BASE_, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0xB8000000},
	{"swr", 2, PARAM_RT_NOBASE, TARGET_R3000_R5900, /*TYPESET_BTYPE,*/ 0xBB800000},
	{"sync", 0, PARAM_NOTHING, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x0000000F},
	{"sync.l", 0, PARAM_NOTHING, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x0000000F},
	{"sync.p", 0, PARAM_NOTHING, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x0000040F},
	{"syscall", 1, PARAM_CODE, TARGET_R3000_R5900, /*TYPESET_CTYPE,*/ 0x0000000C},
	{"syscall", 0, PARAM_NOTHING, TARGET_R3000_R5900, /*TYPESET_CTYPE,*/ 0x0000000C},
	{"teq", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x00000034},
	{"teqi", 2, PARAM_RS_IMM, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x040C0000},
	{"tge", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x00000030},
	{"tgei", 2, PARAM_RS_IMM, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x04080000},
	{"tgeiu", 2, PARAM_RS_IMM, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x04090000},
	{"tgeu", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x00000031},
	{"tlt", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x00000032},
	{"tlti", 2, PARAM_RS_IMM, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x040A0000},
	{"tltiu", 2, PARAM_RS_IMM, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x040B0000},
	{"tltu", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x00000033},
	{"tne", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x00000036},
	{"tnei", 2, PARAM_RS_IMM, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x040E0000},
	{"xor", 3, PARAM_RD_RS_RT, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000026},
	{"xori", 3, PARAM_RT_RS_IMM, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x38000000},
//[R3000]
//addiu	rt,imm			r3000		itype:000000.00000.rt.imm
//subiu	rt,imm			r3000		itype:000000.00000.rt.imm
//[MACROS]
//addu rd,rs,zero -> move rd,rs
	{"move", 2, PARAM_RD_RS, TARGET_R3000_R5900, /*TYPESET_RTYPE,*/ 0x00000021},
//ori rt,zero,imm -> movi rt,imm
//movi	rt,imm			r3000/r5900 itype:001101.00000.rt.imm
//beq rs,zero,off -> beqz rs,off
	{"beqz", 2, PARAM_RS_OFF, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x10000000},
//bne rs,zero,off -> bnez rs,off
	{"bnez", 2, PARAM_RS_OFF, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x14000000},
	{"subi", 3, PARAM_RT_RS_INV, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x20000000},
	{"subiu", 3, PARAM_RT_RS_INV, TARGET_R3000_R5900, /*TYPESET_ITYPE,*/ 0x24000000},
//[R5900]
	{"div1", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x7000001A},
	{"divu1", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x7000001B},
	{"lq", 3, PARAM_RT_OFF_BASE_, TARGET_R5900, /*TYPESET_BTYPE,*/ 0x78000000},
	{"lq", 2, PARAM_RT_NOBASE, TARGET_R5900, /*TYPESET_BTYPE,*/ 0x7B800000},
	{"madd", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000000},
	{"madd", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000000},
	{"madd1", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000020},
	{"madd1", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000020},
	{"maddu", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000001},
	{"maddu", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000001},
	{"maddu1", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000021},
	{"maddu1", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000021},
	{"mfhi1", 1, PARAM_RD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000010},
	{"mflo1", 1, PARAM_RD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000012},
	{"mfsa", 1, PARAM_RD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x00000028},
	{"mthi1", 1, PARAM_RS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000011},
	{"mtlo1", 1, PARAM_RS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000013},
	{"mtsa", 1, PARAM_RS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x00000029},
	{"mtsab", 2, PARAM_RS_IMM, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x04180000},
	{"mtsah", 2, PARAM_RS_IMM, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x04190000},
	{"mult", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x00000018},
	{"mult", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x00000018},
	{"mult1", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000018},
	{"mult1", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000018},
	{"multu", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x00000019},
	{"multu", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x00000019},
	{"multu1", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000019},
	{"multu1", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000019},
	{"pabsh", 2, PARAM_RD_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000168},
	{"pabsw", 2, PARAM_RD_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000068},
	{"paddb", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000208},
	{"paddh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000108},
	{"paddsb", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000608},
	{"paddsh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000508},
	{"paddsw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000408},
	{"paddub", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000628},
	{"padduh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000528},
	{"padduw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000428},
	{"paddw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000008},
	{"padsbh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000128},
	{"pand", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000489},
	{"pceqb", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700002A8},
	{"pceqh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700001A8},
	{"pceqw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700000A8},
	{"pcgtb", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000288},
	{"pcgth", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000188},
	{"pcgtw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000088},
	{"pcpyh", 2, PARAM_RD_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700006E9},
	{"pcpyld", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000389},
	{"pcpyud", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700003A9},
	{"pdivbw", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000749},
	{"pdivuw", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000369},
	{"pdivw", 2, PARAM_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000349},
	{"pexch", 2, PARAM_RD_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700006A9},
	{"pexcw", 2, PARAM_RD_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700007A9},
	{"pexeh", 2, PARAM_RD_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000689},
	{"pexew", 2, PARAM_RD_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000789},
	{"pext5", 2, PARAM_RD_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000788},
	{"pextlb", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000688},
	{"pextlh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000588},
	{"pextlw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000488},
	{"pextub", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700006A8},
	{"pextuh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700005A8},
	{"pextuw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700004A8},
	{"phmadh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000449},
	{"phmsbh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000549},
	{"pinth", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000289},
	{"pinteh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700002A9},
	{"plzcw", 2, PARAM_RD_RS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000004},
	{"pmaddh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000409},
	{"pmadduw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000029},
	{"pmaddw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000009},
	{"pmaxh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700001C8},
	{"pmaxw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700000C8},
	{"pmfhi", 1, PARAM_RD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000209},
	{"pmfhl.lw", 1, PARAM_RD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000030},
	{"pmfhl.uw", 1, PARAM_RD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000070},
	{"pmfhl.slw", 1, PARAM_RD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700000B0},
	{"pmfhl.lh", 1, PARAM_RD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700000F0},
	{"pmfhl.sh", 1, PARAM_RD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000130},
	{"pmflo", 1, PARAM_RD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000249},
	{"pminh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700001E8},
	{"pminw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700000E8},
	{"pmsubh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000509},
	{"pmsubw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000109},
	{"pmthi", 1, PARAM_RS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000229},
	{"pmthl.lw", 1, PARAM_RS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000031},
	{"pmtlo", 1, PARAM_RS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000269},
	{"pmulth", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000709},
	{"pmultuw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000329},
	{"pmultw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000309},
	{"pnor", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700004E9},
	{"por", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700004A9},
	{"ppac5", 2, PARAM_RD_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700007C8},
	{"ppacb", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700006C8},
	{"ppach", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700005C8},
	{"ppacw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700004C8},
	{"prevh", 2, PARAM_RD_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700006C9},
	{"prot3w", 2, PARAM_RD_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700007C9},
	{"psllh", 3, PARAM_RD_RT_SA, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000034},
	{"psllvw", 3, PARAM_RD_RT_RS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000089},
	{"psllw", 3, PARAM_RD_RT_SA, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x7000003C},
	{"psrah", 3, PARAM_RD_RT_SA, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000037},
	{"psravw", 3, PARAM_RD_RT_RS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700000E9},
	{"psraw", 3, PARAM_RD_RT_SA, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x7000003F},
	{"psrlh", 3, PARAM_RD_RT_SA, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000036},
	{"psrlvw", 3, PARAM_RD_RT_RS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700000C9},
	{"psrlw", 3, PARAM_RD_RT_SA, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x7000003E},
	{"psubb", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000248},
	{"psubh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000148},
	{"psubsb", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000648},
	{"psubsh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000548},
	{"psubsw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000448},
	{"psubub", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000668},
	{"psubuh", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000568},
	{"psubuw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000468},
	{"psubw", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x70000048},
	{"pxor", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700004C9},
	{"qfsrv", 3, PARAM_RD_RS_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x700006E8},
	{"sq", 3, PARAM_RT_OFF_BASE_, TARGET_R5900, /*TYPESET_BTYPE,*/ 0x7C000000},
	{"sq", 2, PARAM_RT_NOBASE, TARGET_R5900, /*TYPESET_BTYPE,*/ 0x7F800000},
//[COP0]
	{"bc0f", 1, PARAM_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x41000000},
	{"bc0fl", 1, PARAM_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x41020000},
	{"bc0t", 1, PARAM_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x41010000},
	{"bc0tl", 1, PARAM_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x41030000},
	{"cache", 3, PARAM_OP_OFF_BASE_, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xBC000000},
	{"cache", 2, PARAM_OP_NOBASE, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xBF800000},
	{"di", 0, PARAM_NOTHING, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x42000039},
	{"ei", 0, PARAM_NOTHING, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x42000038},
	{"eret", 0, PARAM_NOTHING, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x42000018},
	{"mfbpc", 1, PARAM_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4000C000},
	{"mfc0", 2, PARAM_RT_RD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x40000000},
	{"mfdab", 1, PARAM_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4000C004},
	{"mfdabm", 1, PARAM_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4000C005},
	{"mfdvb", 1, PARAM_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4000C006},
	{"mfdvbm", 1, PARAM_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4000C007},
	{"mfiab", 1, PARAM_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4000C002},
	{"mfiabm", 1, PARAM_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4000C003},
	{"mfpc", 2, PARAM_RT_REG, TARGET_R5900, /*TYPESET_ETYPE,*/ 0x4000C801},
	{"mfps", 2, PARAM_RT_REG, TARGET_R5900, /*TYPESET_ETYPE,*/ 0x4000C800},
	{"mtbpc", 1, PARAM_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4080C000},
	{"mtc0", 2, PARAM_RT_RD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x40800000},
	{"mtdab", 1, PARAM_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4080C004},
	{"mtdabm", 1, PARAM_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4080C005},
	{"mtbpc", 1, PARAM_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4080C000},
	{"mtdvb", 1, PARAM_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4080C006},
	{"mtdvbm", 1, PARAM_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4080C007},
	{"mtiab", 1, PARAM_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4080C002},
	{"mtiabm", 1, PARAM_RT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4080C003},
	{"mtpc", 2, PARAM_RT_REG, TARGET_R5900, /*TYPESET_ETYPE,*/ 0x4080C801},
	{"mtps", 2, PARAM_RT_REG, TARGET_R5900, /*TYPESET_ETYPE,*/ 0x4080C800},
	{"tlbp", 0, PARAM_NOTHING, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x42000008},
	{"tlbr", 0, PARAM_NOTHING, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x42000001},
	{"tlbwi", 0, PARAM_NOTHING, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x42000002},
	{"tlbwr", 0, PARAM_NOTHING, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x42000006},
//[COP1/FPU]
	{"abs.s", 2, PARAM_FD_FS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000005},
	{"add.s", 3, PARAM_FD_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000000},
	{"adda.s", 2, PARAM_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000018},
	{"bc1f", 1, PARAM_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x45000000},
	{"bc1fl", 1, PARAM_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x45020000},
	{"bc1t", 1, PARAM_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x45010000},
	{"bc1tl", 1, PARAM_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x45030000},
	{"c.f.s", 2, PARAM_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000030},
	{"c.eq.s", 2, PARAM_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000032},
	{"c.lt.s", 2, PARAM_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000034},
	{"c.le.s", 2, PARAM_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000036},
	{"cfc1", 2, PARAM_RT_FS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x44400000},
	{"ctc1", 2, PARAM_RT_FS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x44C00000},
	{"cvt.s.w", 2, PARAM_FD_FS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46800020},
	{"cvt.w.s", 2, PARAM_FD_FS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000024},
	{"div.s", 3, PARAM_FD_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000003},
	{"lwc1", 3, PARAM_FT_OFF_BASE_, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xC4000000},
	{"lwc1", 2, PARAM_FT_NOBASE, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xC7800000},
	{"madd.s", 3, PARAM_FD_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4600001C},
	{"madda.s", 2, PARAM_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4600001E},
	{"max.s", 3, PARAM_FD_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000028},
	{"mfc1", 2, PARAM_RT_FS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x44000000},
	{"min.s", 3, PARAM_FD_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000030},
	{"mov.s", 2, PARAM_FD_FS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000006},
	{"msub.s", 3, PARAM_FD_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4600001D},
	{"msuba.s", 2, PARAM_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4600001F},
	{"mtc1", 2, PARAM_RT_FS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x44800000},
	{"mul.s", 3, PARAM_FD_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000002},
	{"mula.s", 2, PARAM_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x4600001A},
	{"neg.s", 2, PARAM_FD_FS, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000007},
	{"rsqrt.s", 3, PARAM_FD_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000016},
	{"sqrt.s", 2, PARAM_FD_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000004},
	{"sub.s", 3, PARAM_FD_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000001},
	{"suba.s", 2, PARAM_FS_FT, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x46000019},
	{"swc1", 3, PARAM_FT_OFF_BASE_, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xE4000000},
	{"swc1", 2, PARAM_FT_NOBASE, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xE7800000},
//[COP2]
	{"bc2f", 1, PARAM_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x49000000},
	{"bc2fl", 1, PARAM_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x49020000},
	{"bc2t", 1, PARAM_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x49010000},
	{"bc2tl", 1, PARAM_OFF, TARGET_R5900, /*TYPESET_ITYPE,*/ 0x49030000},
	{"cfc2", 2, PARAM_RT_ID, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x48400000},
	{"cfc2.ni", 2, PARAM_RT_ID, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x48400000},
	{"cfc2.i", 2, PARAM_RT_ID, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x48400001},
	{"ctc2", 2, PARAM_RT_ID, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x48C00000},
	{"ctc2.ni", 2, PARAM_RT_ID, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x48C00000},
	{"ctc2.i", 2, PARAM_RT_ID, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x48C00001},
	{"lqc2", 3, PARAM_FT_OFF_BASE_, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xD8000000},
	{"lqc2", 2, PARAM_FT_NOBASE, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xDB800000},
	{"qmfc2", 2, PARAM_RT_FD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x48200000},
	{"qmfc2.ni", 2, PARAM_RT_FD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x48200000},
	{"qmfc2.i", 2, PARAM_RT_FD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x48200001},
	{"qmtc2", 2, PARAM_RT_FD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x48A00000},
	{"qmtc2.ni", 2, PARAM_RT_FD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x48A00000},
	{"qmtc2.i", 2, PARAM_RT_FD, TARGET_R5900, /*TYPESET_RTYPE,*/ 0x48A00001},
	{"sqc2", 3, PARAM_FT_OFF_BASE_, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xF8000000},
	{"sqc2", 2, PARAM_FT_NOBASE, TARGET_R5900, /*TYPESET_BTYPE,*/ 0xFB800000},
//[R3000 COP]
	{"cfc1", 2, PARAM_RT_RD, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x44400000},
	{"cfc2", 2, PARAM_RT_RD, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x48400000},
	{"cfc3", 2, PARAM_RT_RD, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4C400000},
	{"cop0", 1, PARAM_COFUN, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x42000000},
	{"cop1", 1, PARAM_COFUN, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000000},
	{"cop2", 1, PARAM_COFUN, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4A000000},
	{"cop3", 1, PARAM_COFUN, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4E000000},
	{"ctc1", 2, PARAM_RT_RD, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x44C00000},
	{"ctc2", 2, PARAM_RT_RD, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x48C00000},
	{"ctc3", 2, PARAM_RT_RD, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4CC00000},
	{"lwc1", 3, PARAM_RT_OFF_BASE_, TARGET_R3000, /*TYPESET_RTYPE,*/ 0xC4000000},
	{"lwc1", 2, PARAM_RT_NOBASE, TARGET_R3000, /*TYPESET_RTYPE,*/ 0xC7800000},
	{"lwc2", 3, PARAM_RT_OFF_BASE_, TARGET_R3000, /*TYPESET_RTYPE,*/ 0xC8000000},
	{"lwc2", 2, PARAM_RT_NOBASE, TARGET_R3000, /*TYPESET_RTYPE,*/ 0xCB800000},
	{"lwc3", 3, PARAM_RT_OFF_BASE_, TARGET_R3000, /*TYPESET_RTYPE,*/ 0xCC000000},
	{"lwc3", 2, PARAM_RT_NOBASE, TARGET_R3000, /*TYPESET_RTYPE,*/ 0xCF800000},
	{"mfc0", 2, PARAM_RT_RD, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x40000000},
	{"mfc1", 2, PARAM_RT_RD, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x44000000},
	{"mfc2", 2, PARAM_RT_RD, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x48000000},
	{"mfc3", 2, PARAM_RT_RD, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4C000000},
	{"mtc0", 2, PARAM_RT_RD, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x40800000},
	{"mtc1", 2, PARAM_RT_RD, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x44800000},
	{"mtc2", 2, PARAM_RT_RD, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x48800000},
	{"mtc3", 2, PARAM_RT_RD, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4C800000},
	{"swc1", 3, PARAM_RT_OFF_BASE_, TARGET_R3000, /*TYPESET_RTYPE,*/ 0xE4000000},
	{"swc1", 2, PARAM_RT_NOBASE, TARGET_R3000, /*TYPESET_RTYPE,*/ 0xE7800000},
	{"swc2", 3, PARAM_RT_OFF_BASE_, TARGET_R3000, /*TYPESET_RTYPE,*/ 0xE8000000},
	{"swc2", 2, PARAM_RT_NOBASE, TARGET_R3000, /*TYPESET_RTYPE,*/ 0xEB800000},
	{"swc3", 3, PARAM_RT_OFF_BASE_, TARGET_R3000, /*TYPESET_RTYPE,*/ 0xEC000000},
	{"swc3", 2, PARAM_RT_NOBASE, TARGET_R3000, /*TYPESET_RTYPE,*/ 0xEF800000},
//[R3000 FP]
//fmt, fmt3, mnemonic
//10000, 000, S
//10001, 001, D
//10100, 100, W
	{"abs.s", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000005},
	{"abs.d", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200005},
	{"add.s", 3, PARAM_FD_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000000},
	{"add.d", 3, PARAM_FD_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200000},
	{"add.w", 3, PARAM_FD_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800000},
	{"bc1f", 1, PARAM_OFF, TARGET_R3000, /*TYPESET_ITYPE,*/ 0x45000000},
	{"bc1t", 1, PARAM_OFF, TARGET_R3000, /*TYPESET_ITYPE,*/ 0x45010000},
	{"c.f.s", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000030},
	{"c.un.s", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000031},
	{"c.eq.s", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000032},
	{"c.ueq.s", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000033},
	{"c.olt.s", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000034},
	{"c.ult.s", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000035},
	{"c.ole.s", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000036},
	{"c.ule.s", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000037},
	{"c.sf.s", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000038},
	{"c.ngle.s", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000039},
	{"c.seq.s", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4600003A},
	{"c.ngl.s", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4600003B},
	{"c.lt.s", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4600003C},
	{"c.nge.s", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4600003D},
	{"c.le.s", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4600003E},
	{"c.ngt.s", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4600003F},
	{"c.f.d", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200030},
	{"c.un.d", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200031},
	{"c.eq.d", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200032},
	{"c.ueq.d", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200033},
	{"c.olt.d", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200034},
	{"c.ult.d", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200035},
	{"c.ole.d", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200036},
	{"c.ule.d", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200037},
	{"c.sf.d", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200038},
	{"c.ngle.d", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200039},
	{"c.seq.d", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4620003A},
	{"c.ngl.d", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4620003B},
	{"c.lt.d", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4620003C},
	{"c.nge.d", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4620003D},
	{"c.le.d", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4620003E},
	{"c.ngt.d", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4620003F},
	{"c.f.w", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800030},
	{"c.un.w", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800031},
	{"c.eq.w", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800032},
	{"c.ueq.w", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800033},
	{"c.olt.w", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800034},
	{"c.ult.w", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800035},
	{"c.ole.w", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800036},
	{"c.ule.w", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800037},
	{"c.sf.w", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800038},
	{"c.ngle.w", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800039},
	{"c.seq.w", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4680003A},
	{"c.ngl.w", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4680003B},
	{"c.lt.w", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4680003C},
	{"c.nge.w", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4680003D},
	{"c.le.w", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4680003E},
	{"c.ngt.w", 2, PARAM_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x4680003F},
	{"cfc1", 2, PARAM_RT_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x44400000},
	{"ctc1", 2, PARAM_RT_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x44C00000},
	{"cvt.d.s", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000021},
	{"cvt.d.d", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200021},
	{"cvt.d.w", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800021},
	{"cvt.s.s", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000020},
	{"cvt.s.d", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200020},
	{"cvt.s.w", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800020},
	{"cvt.w.s", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000024},
	{"cvt.w.d", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200024},
	{"cvt.w.w", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800024},
	{"div.s", 3, PARAM_FD_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000003},
	{"div.d", 3, PARAM_FD_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200003},
	{"div.w", 3, PARAM_FD_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800003},
	{"lwc1", 3, PARAM_FT_OFF_BASE_, TARGET_R3000, /*TYPESET_BTYPE,*/ 0xC4000000},
	{"lwc1", 2, PARAM_FT_NOBASE, TARGET_R3000, /*TYPESET_BTYPE,*/ 0xC7800000},
	{"mfc1", 2, PARAM_RT_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x44000000},
	{"mov.s", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000006},
	{"mov.d", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200006},
	{"mov.w", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800006},
	{"mtc1", 2, PARAM_RT_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x44800000},
	{"mul.s", 3, PARAM_FD_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000002},
	{"mul.d", 3, PARAM_FD_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200002},
	{"mul.w", 3, PARAM_FD_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800002},
	{"neg.s", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000007},
	{"neg.d", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200007},
	{"neg.w", 2, PARAM_FD_FS, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800007},
	{"sub.s", 3, PARAM_FD_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46000001},
	{"sub.d", 3, PARAM_FD_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46200001},
	{"sub.w", 3, PARAM_FD_FS_FT, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x46800001},
	{"swc1", 3, PARAM_FT_OFF_BASE_, TARGET_R3000, /*TYPESET_BTYPE,*/ 0xE4000000},
	{"swc1", 2, PARAM_FT_NOBASE, TARGET_R3000, /*TYPESET_BTYPE,*/ 0xE7800000},
//[R3000 CP0]
	{"rfe", 0, PARAM_NOTHING, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x42000010},
	{"tlbp", 0, PARAM_NOTHING, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x42000008},
	{"tlbr", 0, PARAM_NOTHING, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x42000001},
	{"tlbwi", 0, PARAM_NOTHING, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x42000002},
	{"tlbwr", 0, PARAM_NOTHING, TARGET_R3000, /*TYPESET_RTYPE,*/ 0x42000006},
//[END OF FILE]
//[LAST UPDATE:07.06.2001]
//[wiRe]
};


#endif